import { Viewer } from '../viewer';
import { GizmoAxes } from './gizmoAxes';
import { GizmoLoading } from './gizmoLoading';
import { GizmoOrbit } from './gizmoOrbit';
import { GizmoRectangle } from './gizmoRectangle';
import { IMeasure } from './measure/measure';
import { SectionBox } from './sectionBox/sectionBox';
import { GizmoMarkers } from './markers/gizmoMarkers';
import { Camera } from '../camera/camera';
/**
 * Represents a collection of gizmos used for various visualization and interaction purposes within the viewer.
 */
export declare class Gizmos {
    private readonly viewer;
    /**
    * The interface to start and manage measure tool interaction.
    */
    get measure(): IMeasure;
    private readonly _measure;
    /**
     * The section box gizmo.
     */
    readonly section: SectionBox;
    /**
     * The loading indicator gizmo.
     */
    readonly loading: GizmoLoading;
    /**
     * The camera orbit target gizmo.
     */
    readonly orbit: GizmoOrbit;
    /**
     * Rectangle Gizmo used for rectangle selection.
     */
    readonly rectangle: GizmoRectangle;
    /**
     * The axis gizmos of the viewer.
     */
    readonly axes: GizmoAxes;
    /**
     * The interface for adding and managing sprite markers in the scene.
     */
    readonly markers: GizmoMarkers;
    constructor(viewer: Viewer, camera: Camera);
    /**
     * Disposes of all gizmos.
     */
    dispose(): void;
}
